/// <reference types="amqplib" />
import * as amqp from 'amqplib';
import { EventEmitter } from 'events';
import { Channel } from './channel';
import Queue from './queue';
export default class Rabbit extends EventEmitter {
    url: string;
    static STOP_PROPAGATION: {
        stopPropagation: boolean;
    };
    connection: amqp.Connection;
    channel: Channel;
    connected: Promise<any>;
    queues: {
        [s: string]: Queue;
    };
    connecting: boolean;
    prefetch: number;
    replyPattern: boolean;
    prefix: string;
    scheduledPublish: boolean;
    socketOptions: any;
    constructor(url: string, {prefetch, replyPattern, prefix, scheduledPublish, socketOptions}?: {
        prefetch?: number;
        replyPattern?: boolean;
        prefix?: string;
        scheduledPublish?: boolean;
        socketOptions?: {};
    });
    private connect();
    reconnect(): Promise<void>;
    private emitDisconnected(error);
    createChannel(connection: amqp.Connection): Promise<amqp.Channel>;
    initChannel(channel: Channel): Promise<void>;
    private updateName(name, prefix?);
    createQueue(name: string, options?: amqp.Options.AssertQueue & {
        prefix?: string;
    }, handler?: (msg: any, ack: (error?, reply?) => any) => any): Promise<Queue>;
    destroyQueue(name: string, prefix?: string): Promise<void>;
    subscribe(name: any, handler: (msg: any, ack: (reply) => any) => any, prefix?: string): Promise<void>;
    unsubscribe(name: any, prefix?: string): Promise<void>;
    publish(name: string, obj: any, headers?: amqp.Options.Publish, prefix?: string): Promise<void>;
    publishWithDelay(name: string, obj: any, properties?: amqp.Options.Publish, prefix?: string): Promise<void>;
    getReply(name: string, obj: any, properties: amqp.Options.Publish, prefix?: string, timeout?: number): Promise<any>;
    getTopicReply(topicName: string, content: any, properties: amqp.Options.Publish, prefix?: string, timeout?: number): Promise<any>;
    publishExchange(exchange: string, routingKey: string, content: any, headers: amqp.Options.Publish, prefix?: string): Promise<void>;
    publishTopic(topicName: string, content: any, headers?: amqp.Options.Publish, prefix?: string): Promise<void>;
    bindToExchange(queueName: string, exchange: string, routingKey: string, prefix?: string): Promise<void>;
    unbindFromExchange(queueName: string, exchange: any, topicName: any, prefix?: string): Promise<void>;
    bindToTopic(queueName: string, topicName: string, prefix?: string): Promise<void>;
    unbindFromTopic(queueName: string, topicName: string, prefix?: string): Promise<void>;
    close(): Promise<void>;
}
